//The originator of the source is okazaki-san.

//
//May.5.2005 Tak.Sugawara
//Dhrystone OPENRISC requires 32KB memory! 
//Use $convert_hex2ver

`define RTL //
module onchip_ram_top (wb_clk_i,
		       wb_rst_i,
		       wb_dat_i,
		       wb_dat_o,
		       wb_adr_i,
		       wb_sel_i,
		       wb_we_i,
		       wb_cyc_i,
		       wb_stb_i,
		       wb_ack_o,
		       wb_err_o);
   //
   // Parameters
   //
   parameter aw = 13;//TAK 12;

   //
   // I/O Ports
   //
   input     wb_clk_i;
   input     wb_rst_i;

   //
   // WB slave i/f
   //
   input [31:0] wb_dat_i;
   output [31:0] wb_dat_o;
   input [31:0]  wb_adr_i;
   input [3:0] 	 wb_sel_i;
   input 	 wb_we_i;
   input 	 wb_cyc_i;
   input 	 wb_stb_i;
   output 	 wb_ack_o;
   output 	 wb_err_o;

   //
   // Internal regs and wires
   //
   wire 	 we;
   wire [3:0] 	 be_i;
   //   wire [aw-1:0] adr;
   wire [31:0] 	 wb_dat_o;
   reg 		 ack_we;
   reg 		 ack_re;

   //
   // Aliases and simple assignments
   //
   assign 	 wb_ack_o = ack_re | ack_we;
   assign 	 wb_err_o = wb_cyc_i & wb_stb_i & (|wb_adr_i[23:aw+2]);

   // If Access to > (8-bit leading prefix ignored)
   assign 	 we = wb_cyc_i & wb_stb_i & wb_we_i & (|wb_sel_i[3:0]);
   assign 	 be_i = (wb_cyc_i & wb_stb_i) * wb_sel_i;

   //
   // Write acknowledge
   //
   always @ (negedge wb_clk_i or posedge wb_rst_i) begin
      if (wb_rst_i)
	ack_we <= 1'b0;
      else if (wb_cyc_i & wb_stb_i & wb_we_i & ~ack_we)
	ack_we <= #1 1'b1;
      else
	ack_we <= #1 1'b0;
   end

   //
   // read acknowledge
   //
   always @ (posedge wb_clk_i or posedge wb_rst_i) begin
      if (wb_rst_i)
	ack_re <= 1'b0;
      else if (wb_cyc_i & wb_stb_i & ~wb_err_o & ~wb_we_i & ~ack_re)
	ack_re <= #1 1'b1;
      else
	ack_re <= #1 1'b0;
   end

  reg [7:0] mem [0:2048*16-1];//Byte Addressing memory
  reg  [8*256:1] ram_initf;//name string buffer
  integer i,j;
  parameter lpm_file="hello.ihex";
	initial begin
	     $convert_hex2ver(lpm_file,8, ram_initf);//Convert Intel Hex File to verilog format
       $readmemh(ram_initf, mem);//Read Verilog Format File
			 j=0;
			 for (i=0;i< 2048*16;i=i+4) begin
				lpm_ram_dq_component_bank0.mem[j]=mem[0+i];//Split into Byte
			  lpm_ram_dq_component_bank1.mem[j]=mem[1+i];//Split into Byte
			  lpm_ram_dq_component_bank2.mem[j]=mem[2+i];//Split into Byte
		 	  lpm_ram_dq_component_bank3.mem[j]=mem[3+i];//Split into Byte
			  j=j+1;
		end	
	end


`ifdef RTL
   lpm_ram_dq_bank0 lpm_ram_dq_component_bank0(
					       .reset(wb_rst_i),
					       .address (wb_adr_i[aw+1:2]),
  					       .inclock (wb_clk_i),
					       .data (wb_dat_i[31:24]),
					       .we (we & be_i[3]),
					       .q (wb_dat_o[31:24]));

   lpm_ram_dq_bank1 lpm_ram_dq_component_bank1(
					       .reset(wb_rst_i),
					       .address (wb_adr_i[aw+1:2]),
					       .inclock (wb_clk_i),
					       .data (wb_dat_i[23:16]),
					       .we (we & be_i[2]),
					       .q (wb_dat_o[23:16]));
   
   lpm_ram_dq_bank2 lpm_ram_dq_component_bank2(
					       .reset(wb_rst_i),
					       .address (wb_adr_i[aw+1:2]),
					       .inclock (wb_clk_i),
					       .data (wb_dat_i[15:8]),
					       .we (we & be_i[1]),
					       .q (wb_dat_o[15:8]));

   lpm_ram_dq_bank3 lpm_ram_dq_component_bank3(
					       .reset(wb_rst_i),
					       .address (wb_adr_i[aw+1:2]),
					       .inclock (wb_clk_i),
					       .data (wb_dat_i[7:0]),
					       .we (we & be_i[0]),
					       .q (wb_dat_o[7:0]));

`else				// altera_mf.v
    onchip_ram0 lpm_ram_dq_component_bank0(
					          .address (wb_adr_i[aw+1:2]),
  					       .clock (wb_clk_i),
					       .data (wb_dat_i[31:24]),
					       .wren (we & be_i[3]),
					       .q (wb_dat_o[31:24]));
 onchip_ram1 lpm_ram_dq_component_bank1(
					       .address (wb_adr_i[aw+1:2]),
					       .clock (wb_clk_i),
					       .data (wb_dat_i[23:16]),
					       .wren (we & be_i[2]),
					       .q (wb_dat_o[23:16]));

  onchip_ram2 lpm_ram_dq_component_bank2(
					       .address (wb_adr_i[aw+1:2]),
					       .clock (wb_clk_i),
					       .data (wb_dat_i[15:8]),
					       .wren (we & be_i[1]),
					       .q (wb_dat_o[15:8]));

  onchip_ram3 lpm_ram_dq_component_bank3(
					       .address (wb_adr_i[aw+1:2]),
					       .clock (wb_clk_i),
					       .data (wb_dat_i[7:0]),
					       .wren (we & be_i[0]),
					       .q (wb_dat_o[7:0]));
`endif //  `ifdef RTL


`ifdef Veritak
	`define Print_Port_Address      16'h7ff0  //Debug Console Port
      `define Print_CAHR_Port_Address 16'h7ff1  //
      `define Print_INT_Port_Address  16'h7ff2  //
      `define Print_LONG_Port_Address 16'h7ff4  //
	

   reg [0:640*2-1] char_buffer;
   integer  counter=0;    
   wire mem_write=wb_we_i;
   wire clock=wb_clk_i;
   wire [31:0] mem_address=wb_adr_i;
   wire [31:0] mem_data_w=wb_dat_i;
   
   task Cprint;// String OUT until the byte 00 or xx detected with least Byte first and justified.
		integer i;
		begin :Block
			i=0;
			while (1) begin
				if (char_buffer[i*8 +:8] ===8'h00 || char_buffer[i*8 +:8]===8'hxx) begin
						disable Block;
				end	
				$write("%c",char_buffer[i*8 +:8]);
				i=i+1;  
			end
		end
	endtask	
   always @ (negedge clock ) begin
            if ((mem_write === 1'b1)) begin 
           	   if (mem_address==`Print_Port_Address) begin
				if (mem_data_w[7:0]===8'h00) begin
					char_buffer[counter*8 +:8]=mem_data_w[7:0];
					if (char_buffer[0  +:8*7]==="$finish") begin
							$stop;				
							
					end else if (char_buffer[0  +:8*5]==="$time") begin
							$display("Current Time on Simulation=%d",$time);				
							
					end else  Cprint;//$write("%s",char_buffer);

					for (counter=0; counter< 80*2; counter=counter+1) begin
						char_buffer[counter*8 +:8]=8'h00;
					end
					counter=0;
				
				end else begin
					char_buffer[counter*8 +:8]=mem_data_w[7:0];
					counter=counter+1;
				end
		   end
       	   else if (mem_address==`Print_CAHR_Port_Address) begin
				$write("%h ",mem_data_w[7:0]);
		   end else if (mem_address==`Print_INT_Port_Address) begin
				$write("%h ",mem_data_w[15:0]);//Little Endian 
		   end else if (mem_address==`Print_LONG_Port_Address) begin
				$write("%h ",mem_data_w[31:0]);//Big Endian
		   end 
	end //if
   end //always
`endif


endmodule // onchip_ram_top

`ifdef RTL
module lpm_ram_dq_bank0(
			reset,
			address,
			we,
			inclock,
			data,
			q);
   input                reset;
   input [12:0] 	address;
   input 		we;
   input 		inclock;
   input [7:0] 		data;
   output [7:0] 	q;

   reg [7:0] 		mem[0:4096*2-1];//TAK
   
   //    assign 		q = we ? 8'hZZ : (inclock ? mem[address] : 8'hZZ);
   assign 		q = we ? 8'hZZ : mem[address];
   always @(posedge inclock or posedge reset) begin
      if (reset) begin
 //`include "onchip_ram_bank0.v"
      end else if (we == 1'b1) begin
	 mem[address] <= data;
      end
   end
endmodule

module lpm_ram_dq_bank1(
			reset,
			address,
			we,
			inclock,
			data,
			q);
   input                reset;
   input [12:0] 	address;
   input 		we;
   input 		inclock;
   input [7:0] 		data;
   output [7:0] 	q;

   reg [7:0] 		mem[0:4096*2-1];
   
   assign 		q = we ? 8'hZZ : mem[address];
   always @(posedge inclock or posedge reset) begin
      if (reset) begin
 //`include "onchip_ram_bank1.v"
      end else if (we == 1'b1) begin
	 mem[address] <= data;
      end
   end
endmodule

module lpm_ram_dq_bank2(
			reset,
			address,
			we,
			inclock,
			data,
			q);
   input                reset;
   input [12:0] 	address;
   input 		we;
   input 		inclock;
   input [7:0] 		data;
   output [7:0] 	q;

   reg [7:0] 		mem[0:4096*2-1];
   
   assign 		q = we ? 8'hZZ : mem[address];
   always @(posedge inclock or posedge reset) begin
      if (reset) begin
// `include "onchip_ram_bank2.v"
      end else if (we == 1'b1) begin
	 mem[address] <= data;
      end
   end
endmodule

module lpm_ram_dq_bank3(
			reset,
			address,
			we,
			inclock,
			data,
			q);
   input                reset;
   input [12:0] 	address;
   input 		we;
   input 		inclock;
   input [7:0] 		data;
   output [7:0] 	q;

   reg [7:0] 		mem[0:4096*2-1];
   
   assign 		q = we ? 8'hZZ : mem[address];
   always @(posedge inclock or posedge reset) begin
      if (reset) begin
// `include "onchip_ram_bank3.v"
      end else if (we == 1'b1) begin
	 mem[address] <= data;
      end
   end

endmodule
`endif


